###############################################################################
# Import Python modules
import arcpy
import numpy

###############################################################################
# Get parameter inputs
dt_sum_table = arcpy.GetParameterAsText(0)
dt_map = arcpy.GetParameterAsText(1)
dt_id_field = arcpy.GetParameterAsText(2)
dt_area_field = arcpy.GetParameterAsText(3)
ds_map = arcpy.GetParameterAsText(4)
ds_id_field = arcpy.GetParameterAsText(5)
ds_area_field = arcpy.GetParameterAsText(6)
output_map = arcpy.GetParameterAsText(7)
it_id_field = arcpy.GetParameterAsText(8)
it_area_field = arcpy.GetParameterAsText(9)
prop_field = arcpy.GetParameterAsText(10)
extra_field_str = arcpy.GetParameterAsText(11)

###############################################################################
# Check unique ID for data
dt_id = []
cursor = arcpy.SearchCursor(dt_map)
for row in cursor:
  dt_id.append(row.getValue(dt_id_field))
if len(dt_id) == len(set(dt_id)):
  arcpy.AddMessage("All data IDs are unique.")
else:
  arcpy.AddMessage("ERROR: Not all data IDs are unique.")
  sys.exit()

###############################################################################
# Check if data area field exists
if arcpy.ListFields(dt_map, dt_area_field):
  arcpy.AddMessage("Data area field exists.")
  arcpy.AddMessage("Deleting data area field...")
  arcpy.DeleteField_management(dt_map, dt_area_field) 
else:
  arcpy.AddMessage("Data area field does not exist.")
arcpy.AddMessage("Adding data area field...")
arcpy.AddField_management(dt_map, dt_area_field, "DOUBLE")

###############################################################################
# Get data area
arcpy.AddMessage("Calculating data area...")
arcpy.CalculateField_management(dt_map, dt_area_field,
  "!SHAPE.AREA@SQUAREKILOMETERS!", "PYTHON_9.3")
dt_area = []
cursor = arcpy.SearchCursor(dt_map)
for row in cursor:
  dt_area.append(row.getValue(dt_area_field))

###############################################################################
# Check unique ID for distribution
ds_id = []
cursor = arcpy.SearchCursor(ds_map)
for row in cursor:
  ds_id.append(row.getValue(ds_id_field))
if len(ds_id) == len(set(ds_id)):
  arcpy.AddMessage("All distribution IDs are unique.")
else:
  arcpy.AddMessage("ERROR: Not all distribution IDs are unique.")
  sys.exit()

###############################################################################
# Check if distribution area field exists
if arcpy.ListFields(ds_map, ds_area_field):
  arcpy.AddMessage("Distribution area field exists.")
  arcpy.AddMessage("Deleting distribution area field...")
  arcpy.DeleteField_management(ds_map, ds_area_field) 
else:
  arcpy.AddMessage("Distribution area field does not exist.")
arcpy.AddMessage("Adding distribution area field...")
arcpy.AddField_management(ds_map, ds_area_field, "DOUBLE")

###############################################################################
# Get distribution area
arcpy.AddMessage("Calculating distribution area...")
arcpy.CalculateField_management(ds_map, ds_area_field,
  "!SHAPE.AREA@SQUAREKILOMETERS!", "PYTHON_9.3")
ds_area = []
cursor = arcpy.SearchCursor(ds_map)
for row in cursor:
  ds_area.append(row.getValue(ds_area_field))

###############################################################################
# Get data sum table field names
sum_field_names = []
fields = arcpy.ListFields(dt_sum_table)
for field in fields:
  if field.name != dt_id_field:
    sum_field_names.append(field.name)
arcpy.AddMessage("Data sum fields to be processed: " + \
  ", ".join(sum_field_names))
ds_field_names = []
fields = arcpy.ListFields(ds_map)
for field in fields:
  ds_field_names.append(field.name)
if all(i in ds_field_names for i in sum_field_names):
  arcpy.AddMessage("All data sum fields found in distribution map.")
else:
  arcpy.AddMessage("ERROR: Not all data sum fields found in distribution map.")
  sys.exit()

###############################################################################
# Check field types
sum_field_types = []
fields = arcpy.ListFields(dt_sum_table)
for field in fields:
  if field.name != dt_id_field:
    sum_field_types.append(field.type)
if all(i == "Double" for i in sum_field_types):
  arcpy.AddMessage("All data sum field types are double.")
else:
  arcpy.AddMessage("ERROR: Not all data sum field types are double.")
  sys.exit()
ds_field_types = []
fields = arcpy.ListFields(ds_map)
for field in fields:
  if field.name in sum_field_names:
    ds_field_types.append(field.type)
if all(i == "Double" for i in ds_field_types):
  arcpy.AddMessage("All distribution field types are double.")
else:
  arcpy.AddMessage("ERROR: Not all distribution field types are double.")
  sys.exit()

###############################################################################
# Check data sum table ID
dt_id_in_sum_table = []
cursor = arcpy.SearchCursor(dt_sum_table)
for row in cursor:
  dt_id_in_sum_table.append(row.getValue(dt_id_field))
if len(dt_id) == len(dt_id_in_sum_table) and \
  all([i == j for i, j in zip(dt_id, dt_id_in_sum_table)]):
  arcpy.AddMessage("IDs in data sum table match with data IDs.")
else:
  arcpy.AddMessage("ERROR: IDs in data sum table do not match with data IDs.")
  sys.exit()

###############################################################################
# Read data sum table
dt_sum = []
for i in sum_field_names:
  dt_sum.append([])
cursor = arcpy.SearchCursor(dt_sum_table)
for row in cursor:
  for i in range(len(sum_field_names)):
    dt_sum[i].append(row.getValue(sum_field_names[i]))

###############################################################################
# Perform intersection
arcpy.AddMessage("Intersecting data and distribution maps...")
in_features = []
in_features.append(dt_map)
in_features.append(ds_map)
arcpy.Intersect_analysis(in_features, output_map)
mxd = arcpy.mapping.MapDocument("CURRENT")
df = arcpy.mapping.ListDataFrames(mxd,"*")[0]
newlayer = arcpy.mapping.Layer(output_map)
arcpy.mapping.AddLayer(df, newlayer,"TOP")

###############################################################################
# Check if intersect area field exists
if arcpy.ListFields(output_map, it_area_field):
  arcpy.AddMessage("Intersect area field exists.")
  arcpy.AddMessage("Deleting intersect area field...")
  arcpy.DeleteField_management(output_map, it_area_field) 
else:
  arcpy.AddMessage("Intersect area field does not exist.")
arcpy.AddMessage("Adding intersect area field...")
arcpy.AddField_management(output_map, it_area_field, "DOUBLE")

###############################################################################
# Calculate intersect area
arcpy.AddMessage("Calculating intersect area...")
arcpy.CalculateField_management(output_map, it_area_field,
  "!SHAPE.AREA@SQUAREKILOMETERS!", "PYTHON_9.3")

###############################################################################
# Check if proportion field exists
if arcpy.ListFields(output_map, prop_field):
  arcpy.AddMessage("Proportion field exists.")
  arcpy.AddMessage("Deleting proportion field...")
  arcpy.DeleteField_management(output_map, prop_field) 
else:
  arcpy.AddMessage("Proportion field does not exist.")
arcpy.AddMessage("Adding proportion field...")
arcpy.AddField_management(output_map, prop_field, "DOUBLE")

###############################################################################
# Calculate proportion
arcpy.AddMessage("Calculating proportion...")
arcpy.CalculateField_management(output_map, prop_field,
  "!" + it_area_field + "!" +"/" + "!" + ds_area_field + "!", "PYTHON_9.3")

###############################################################################
# Calculate sum of intersect areas
arcpy.AddMessage("Distributing data in sum table...")
it_area_sum = numpy.zeros((len(dt_id), len(sum_field_names)))
it_cnt_sum = numpy.zeros((len(dt_id), len(sum_field_names)))
cursor = arcpy.SearchCursor(output_map)
for row in cursor:
  dt_value = row.getValue(dt_id_field)
  dt_index = dt_id.index(dt_value)
  prop_value = row.getValue(prop_field)
  for i in range(len(sum_field_names)): 
    ds_value = row.getValue(sum_field_names[i])
    it_area_sum[dt_index, i] = it_area_sum[dt_index, i] + \
      ds_value * prop_value
    it_cnt_sum[dt_index, i] = it_cnt_sum[dt_index, i] + \
      prop_value

###############################################################################
# Distribute data
cursor = arcpy.UpdateCursor(output_map)
for row in cursor:
  dt_value = row.getValue(dt_id_field)
  dt_index = dt_id.index(dt_value)
  prop_value = row.getValue(prop_field)
  for i in range(len(sum_field_names)):
    ds_value = row.getValue(sum_field_names[i])
    if it_area_sum[dt_index, i] == 0:
      new_value = \
        dt_sum[i][dt_index] / it_cnt_sum[dt_index, i] * prop_value
    else:
      new_value = \
        dt_sum[i][dt_index] * ds_value / it_area_sum[dt_index, i] * prop_value
    row.setValue(sum_field_names[i], new_value)
    cursor.updateRow(row)

###############################################################################
# Distribute extra fields
if not extra_field_str:
  arcpy.AddMessage("No extra distribution field selected.")
else:
  extra_field = extra_field_str.split(";")
  arcpy.AddMessage("Extra output selected: " + ", ".join(extra_field))
  arcpy.AddMessage("Distributing data in extra fields...")
  extra_field_types = []
  fields = arcpy.ListFields(output_map)
  for field in fields:
    if field.name in extra_field:
      extra_field_types.append(field.type)
      cursor = arcpy.UpdateCursor(output_map)
      for row in cursor:
        prop_value = row.getValue(prop_field)
        new_value = row.getValue(field.name) * prop_value
        row.setValue(field.name, new_value)
        cursor.updateRow(row)
  if all(i == "Double" for i in extra_field_types):
    arcpy.AddMessage("All extra field types are double.")
  else:
    arcpy.AddMessage("ERROR: Not all extra field types are double.")
    sys.exit()

###############################################################################
# Check if output ID field exists
if arcpy.ListFields(output_map, it_id_field):
  arcpy.AddMessage("Output ID field exists.")
  arcpy.AddMessage("Deleting output ID field...")
  arcpy.DeleteField_management(output_map, it_id_field) 
else:
  arcpy.AddMessage("Output ID field does not exist.")
arcpy.AddMessage("Adding output ID field field...")
arcpy.AddField_management(output_map, it_id_field, "LONG")

###############################################################################
# Add output ID
arcpy.AddMessage("Calculating output ID...")
arcpy.CalculateField_management(output_map, it_id_field, "!FID! + 1",
  "PYTHON_9.3")
