###############################################################################
# Import Python modules
import arcpy
import numpy

###############################################################################
# Get parameter inputs
wt_map = arcpy.GetParameterAsText(0)
wt_id_field = arcpy.GetParameterAsText(1)
wt_area_field = arcpy.GetParameterAsText(2)
dt_map = arcpy.GetParameterAsText(3)
dt_id_field = arcpy.GetParameterAsText(4)
dt_area_field = arcpy.GetParameterAsText(5)
output_table = arcpy.GetParameterAsText(6)
output_map = arcpy.GetParameterAsText(7)
it_area_field = arcpy.GetParameterAsText(8)
prop_field = arcpy.GetParameterAsText(9)
extra_field_str = arcpy.GetParameterAsText(10)

###############################################################################
# Check unique ID for watersheds
wt_id = []
cursor = arcpy.SearchCursor(wt_map)
for row in cursor:
  wt_id.append(row.getValue(wt_id_field))
if len(wt_id) == len(set(wt_id)):
  arcpy.AddMessage("All watershed IDs are unique.")
else:
  arcpy.AddMessage("ERROR: Not all watershed IDs are unique.")
  sys.exit()

###############################################################################
# Check if watershed area field exists
if arcpy.ListFields(wt_map, wt_area_field):
  arcpy.AddMessage("Watershed area field exists.")
  arcpy.AddMessage("Deleting watershed area field...")
  arcpy.DeleteField_management(wt_map, wt_area_field) 
else:
  arcpy.AddMessage("Watershed area field does not exist.")
arcpy.AddMessage("Adding watershed area field...")
arcpy.AddField_management(wt_map, wt_area_field, "DOUBLE")

###############################################################################
# Get watershed area
arcpy.AddMessage("Calculating watershed area...")
arcpy.CalculateField_management(wt_map, wt_area_field,
  "!SHAPE.AREA@SQUAREKILOMETERS!", "PYTHON_9.3")
wt_area = []
cursor = arcpy.SearchCursor(wt_map)
for row in cursor:
  wt_area.append(row.getValue(wt_area_field))

###############################################################################
# Check unique ID for data
dt_id = []
cursor = arcpy.SearchCursor(dt_map)
for row in cursor:
  dt_id.append(row.getValue(dt_id_field))
if len(dt_id) == len(set(dt_id)):
  arcpy.AddMessage("All data IDs are unique.")
else:
  arcpy.AddMessage("ERROR: Not all data IDs are unique.")
  sys.exit()

###############################################################################
# Check if data area field exists
if arcpy.ListFields(dt_map, dt_area_field):
  arcpy.AddMessage("Data area field exists.")
  arcpy.AddMessage("Deleting data area field...")
  arcpy.DeleteField_management(dt_map, dt_area_field) 
else:
  arcpy.AddMessage("Data area field does not exist.")
arcpy.AddMessage("Adding data area field...")
arcpy.AddField_management(dt_map, dt_area_field, "DOUBLE")

###############################################################################
# Get data area
arcpy.AddMessage("Calculating data area...")
arcpy.CalculateField_management(dt_map, dt_area_field,
  "!SHAPE.AREA@SQUAREKILOMETERS!", "PYTHON_9.3")
dt_area = []
cursor = arcpy.SearchCursor(dt_map)
for row in cursor:
  dt_area.append(row.getValue(dt_area_field))

###############################################################################
# Perform intersection
arcpy.AddMessage("Intersecting watershed and data maps...")
in_features = []
in_features.append(wt_map)
in_features.append(dt_map)
arcpy.Intersect_analysis(in_features, output_map)
mxd = arcpy.mapping.MapDocument("CURRENT")
df = arcpy.mapping.ListDataFrames(mxd,"*")[0]
newlayer = arcpy.mapping.Layer(output_map)
arcpy.mapping.AddLayer(df, newlayer,"TOP")

###############################################################################
# Check if intersect area field exists
if arcpy.ListFields(output_map, it_area_field):
  arcpy.AddMessage("Intersect area field exists.")
  arcpy.AddMessage("Deleting intersect area field...")
  arcpy.DeleteField_management(output_map, it_area_field) 
else:
  arcpy.AddMessage("Intersect area field does not exist.")
arcpy.AddMessage("Adding intersect area field...")
arcpy.AddField_management(output_map, it_area_field, "DOUBLE")

###############################################################################
# Calculate intersect area
arcpy.AddMessage("Calculating intersect area...")
arcpy.CalculateField_management(output_map, it_area_field,
  "!SHAPE.AREA@SQUAREKILOMETERS!", "PYTHON_9.3")

###############################################################################
# Check if proportion field exists
if arcpy.ListFields(output_map, prop_field):
  arcpy.AddMessage("Proportion field exists.")
  arcpy.AddMessage("Deleting proportion field...")
  arcpy.DeleteField_management(output_map, prop_field) 
else:
  arcpy.AddMessage("Proportion field does not exist.")
arcpy.AddMessage("Adding proportion field...")
arcpy.AddField_management(output_map, prop_field, "DOUBLE")

###############################################################################
# Calculate proportion
arcpy.AddMessage("Calculating proportion...")
arcpy.CalculateField_management(output_map, prop_field,
  "!" + it_area_field + "!" +"/" + "!" + dt_area_field + "!", "PYTHON_9.3")

###############################################################################
# Filter unique ID for data
it_id = []
cursor = arcpy.SearchCursor(output_map)
for row in cursor:
  it_id.append(row.getValue(dt_id_field))
dt_id_in_it = [i for i in dt_id if i in it_id]
dt_area_in_it = [dt_area[pos] for pos, i in enumerate(dt_id) if i in it_id]

###############################################################################
# Generate Output Table
prop_table = numpy.zeros((len(dt_id_in_it), len(wt_id)))
cursor = arcpy.SearchCursor(output_map)
for row in cursor:
  wt_index = wt_id.index(row.getValue(wt_id_field))
  dt_index = dt_id_in_it.index(row.getValue(dt_id_field))
  prop_table[dt_index, wt_index] = prop_table[dt_index, wt_index] + \
    row.getValue(prop_field)

###############################################################################
# Read additional output field data
if not extra_field_str:
  arcpy.AddMessage("No extra output selected.")
else:
  extra_field = extra_field_str.split(";")
  arcpy.AddMessage("Extra output selected: " + ", ".join(extra_field))
  dt_extra = []
  dt_extra_in_it = []  
  for i in extra_field:
    dt_extra.append([])
    dt_extra_in_it.append([])
  cursor = arcpy.SearchCursor(dt_map)
  for row in cursor:
    for i in range(len(extra_field)):
      dt_extra[i].append(row.getValue(extra_field[i]))
  for i in range(len(extra_field)):
    dt_extra_in_it[i] = \
      [dt_extra[i][pos] for pos, j in enumerate(dt_id) if j in it_id]

###############################################################################
# Write Output Table
arcpy.AddMessage("Writing output table...")
f = open(output_table, "w")
line_str = '"' + dt_id_field + '",' + \
  "{}".format('"' + '","'.join(extra_field) + '",' \
  if extra_field_str else "") + \
  '"Area_km2",' + \
  '"' + '","'.join(wt_id) + '"\n'
f.write(line_str)
line_str = '"Area_km2",' + \
  "{}".format('"' + '","'.join(["-" for i in extra_field]) + '",' \
  if extra_field_str else "") + \
  '"-",' + \
  '"' + '","'.join([str(i) for i in wt_area]) + '"\n'
f.write(line_str)
for i in range(len(dt_id_in_it)):
  line_str = '"' + str(dt_id_in_it[i]) + '",' + \
    "{}".format('"' + '","'.join([dt_extra_in_it[j][i] for j in \
    range(len(extra_field))]) + '",' if extra_field_str else "") + \
    '"' + str(dt_area_in_it[i]) + '",' + \
    '"' + '","'.join([str(prop_table[i][j]) for j in \
    range(len(wt_id))]) + '"\n'
  f.write(line_str)
f.close()

